@@ -1,15 +1,17 @@
 """
 Translation support for admin forms.
 """
+import django
 from django.conf import settings
 from django.conf.urls import patterns, url
 from django.contrib import admin
-from django.contrib.admin.options import csrf_protect_m
+from django.contrib.admin.options import csrf_protect_m, BaseModelAdmin, InlineModelAdmin
 from django.contrib.admin.util import get_deleted_objects, unquote
 from django.core.exceptions import PermissionDenied, ImproperlyConfigured
 from django.core.urlresolvers import reverse
 from django.db import router
 from django.forms import Media
+from django.forms.models import BaseInlineFormSet
 from django.http import HttpResponseRedirect, Http404, HttpRequest
 from django.shortcuts import render
 from django.utils.encoding import iri_to_uri, force_unicode
@@ -38,40 +40,21 @@ _language_prepopulated_media = _language_media + Media(js=(
 _fakeRequest = HttpRequest()
 
 
-class TranslatableAdmin(admin.ModelAdmin):
+class BaseTranslatableAdmin(BaseModelAdmin):
     """
-    Base class for translated admins.
-
-    This class also works as regular admin for non TranslatableModel objects.
-    When using this class with a non-TranslatableModel,
-    all operations effectively become a NO-OP.
+    The shared code between the regular model admin and inline classes.
     """
-
     form = TranslatableModelForm
-
-    deletion_not_allowed_template = 'admin/parler/deletion_not_allowed.html'
-
     query_language_key = 'language'
 
 
-    @property
-    def change_form_template(self):
-        # Dynamic property to support transition to regular models.
-        if self._has_translatable_model():
-            # While this breaks the admin template name detection,
-            # the get_change_form_base_template() makes sure it inherits from your template.
-            return 'admin/parler/change_form.html'
-        else:
-            return None # get default admin selection
-
-
     @property
     def media(self):
         # Currently, `prepopulated_fields` can't be used because it breaks the admin validation.
         # TODO: as a fix TranslatedFields should become a RelatedField on the shared model (may also support ORM queries)
         # As workaround, declare the fields in get_prepopulated_fields() and we'll provide the admin media automatically.
         has_prepoplated = len(self.get_prepopulated_fields(_fakeRequest))
-        base_media = super(TranslatableAdmin, self).media
+        base_media = super(BaseTranslatableAdmin, self).media
         if has_prepoplated:
             return base_media + _language_prepopulated_media
         else:
@@ -109,6 +92,62 @@ class TranslatableAdmin(admin.ModelAdmin):
             return normalize_language_code(code)
 
 
+    def get_form_language(self, request, obj=None):
+        """
+        Return the current language for the currently displayed object fields.
+        """
+        if obj is not None:
+            return obj.get_current_language()
+        else:
+            return self._language(request)
+
+
+    def queryset(self, request):
+        """
+        Make sure the current language is selected.
+        """
+        qs = super(BaseTranslatableAdmin, self).queryset(request)
+
+        if self._has_translatable_model():
+            if not isinstance(qs, TranslatableQuerySet):
+                raise ImproperlyConfigured("{0} class does not inherit from TranslatableQuerySet".format(qs.__class__.__name__))
+
+            if not is_multilingual_project():
+                # Make sure the current translations remain visible, not the dynamically set get_language() value.
+                qs = qs.language(appsettings.PARLER_DEFAULT_LANGUAGE_CODE)
+            else:
+                # Set the initial language for fetched objects.
+                # This is needed for the TranslatableInlineModelAdmin
+                qs = qs.language(self._language(request))
+        return qs
+
+
+class TranslatableAdmin(BaseTranslatableAdmin, admin.ModelAdmin):
+    """
+    Base class for translated admins.
+
+    This class also works as regular admin for non TranslatableModel objects.
+    When using this class with a non-TranslatableModel,
+    all operations effectively become a NO-OP.
+    """
+
+    deletion_not_allowed_template = 'admin/parler/deletion_not_allowed.html'
+
+    #: Whether translations of inlines should also be deleted when deleting a translation.
+    delete_inline_translations = True
+
+
+    @property
+    def change_form_template(self):
+        # Dynamic property to support transition to regular models.
+        if self._has_translatable_model():
+            # While this breaks the admin template name detection,
+            # the get_change_form_base_template() makes sure it inherits from your template.
+            return 'admin/parler/change_form.html'
+        else:
+            return None # get default admin selection
+
+
     def language_column(self, object):
         """
         The language column which can be included in the ``list_display``.
@@ -138,22 +177,6 @@ class TranslatableAdmin(admin.ModelAdmin):
             return self.model._translations_model.objects.get_empty_query_set()
 
 
-    def queryset(self, request):
-        """
-        Make sure the current language is selected.
-        """
-        qs = super(TranslatableAdmin, self).queryset(request)
-
-        if self._has_translatable_model():
-            if not isinstance(qs, TranslatableQuerySet):
-                raise ImproperlyConfigured("{0} class does not inherit from TranslatableQuerySet".format(qs.__class__.__name__))
-
-            if not is_multilingual_project():
-                # Make sure the current translations remain visible, not the dynamically set get_language() value.
-                qs = qs.language(appsettings.PARLER_DEFAULT_LANGUAGE_CODE)
-        return qs
-
-
     def get_object(self, request, object_id):
         """
         Make sure the object is fetched in the correct language.
@@ -176,16 +199,6 @@ class TranslatableAdmin(admin.ModelAdmin):
         return form_class
 
 
-    def get_form_language(self, request, obj=None):
-        """
-        Return the current language for the currently displayed object fields.
-        """
-        if obj is not None:
-            return obj.get_current_language()
-        else:
-            return self._language(request)
-
-
     def get_urls(self):
         """
         Add a delete-translation view.
@@ -233,6 +246,10 @@ class TranslatableAdmin(admin.ModelAdmin):
 
 
     def response_add(self, request, obj, post_url_continue=None):
+        # Minor behavior difference for Django 1.4
+        if post_url_continue is None and django.VERSION < (1,5):
+            post_url_continue = '../%s/'
+
         # Make sure ?language=... is included in the redirects.
         redirect = super(TranslatableAdmin, self).response_add(request, obj, post_url_continue)
         return self._patch_redirect(request, obj, redirect)
@@ -333,6 +350,15 @@ class TranslatableAdmin(admin.ModelAdmin):
         (deleted_objects, perms_needed, protected) = get_deleted_objects(
             [translation], translations_model._meta, request.user, self.admin_site, using)
 
+        # Extend deleted objects with the inlines.
+        if self.delete_inline_translations:
+            shared_obj = translation.master
+            for inline, qs in self._get_inline_translations(request, translation.language_code, obj=shared_obj):
+                (del2, perms2, protected2) = get_deleted_objects(qs, qs.model._meta, request.user, self.admin_site, using)
+                deleted_objects += del2
+                perms_needed = perms_needed or perms2
+                protected += protected2
+
         if request.POST: # The user has already confirmed the deletion.
             if perms_needed:
                 raise PermissionDenied
@@ -395,6 +421,32 @@ class TranslatableAdmin(admin.ModelAdmin):
         """
         translation.delete()
 
+        # Also delete translations of inlines which the user has access to.
+        if self.delete_inline_translations:
+            master = translation.master
+            for inline, qs in self._get_inline_translations(request, translation.language_code, obj=master):
+                qs.delete()
+
+
+    def _get_inline_translations(self, request, language_code, obj=None):
+        """
+        Fetch the inline translations
+        """
+        for inline in self.get_inline_instances(request, obj=obj):
+            if issubclass(inline.model, TranslatableModel):
+                # leverage inlineformset_factory() to find the ForeignKey.
+                # This also resolves the fk_name if it's set.
+                fk = inline.get_formset(request, obj).fk
+
+                rel_name = 'master__{}'.format(fk.name)
+                filters = {
+                    'language_code': language_code,
+                    rel_name: obj
+                }
+
+                qs = inline.model._translations_model.objects.filter(**filters)
+                yield inline, qs
+
 
     def get_change_form_base_template(self):
         """
@@ -410,3 +462,35 @@ class TranslatableAdmin(admin.ModelAdmin):
 
 
 _lazy_select_template_name = lazy(select_template_name, unicode)
+
+
+class TranslatableBaseInlineFormSet(BaseInlineFormSet):
+    language_code = None
+
+    def _construct_form(self, i, **kwargs):
+        form = super(TranslatableBaseInlineFormSet, self)._construct_form(i, **kwargs)
+        form.language_code = self.language_code   # Pass the language code for new objects!
+        return form
+
+    def save_new(self, form, commit=True):
+        obj = super(TranslatableBaseInlineFormSet, self).save_new(form, commit)
+        return obj
+
+
+class TranslatableInlineModelAdmin(BaseTranslatableAdmin, InlineModelAdmin):
+    form = TranslatableModelForm
+    formset = TranslatableBaseInlineFormSet
+
+    def get_formset(self, request, obj=None, **kwargs):
+        FormSet = super(TranslatableInlineModelAdmin, self).get_formset(request, obj, **kwargs)
+        # Existing objects already got the language code from the queryset().language() method.
+        # For new objects, the language code should be set here.
+        FormSet.language_code = self.get_form_language(request, obj)
+        return FormSet
+
+
+class TranslatableStackedInline(TranslatableInlineModelAdmin):
+    template = 'admin/edit_inline/stacked.html'
+
+class TranslatableTabularInline(TranslatableInlineModelAdmin):
+    template = 'admin/edit_inline/tabular.html'
