@@ -314,10 +314,20 @@ class TranslatableModel(models.Model):
         # Save all translated objects which were fetched.
         # This also supports switching languages several times, and save everything in the end.
         for translation in self._translations_cache.itervalues():
-            if translation.is_modified:
-                if not translation.master_id:  # Might not exist during first construction
-                    translation.master = self
-                translation.save()
+            if translation is None:  # Skip fallback markers
+                continue
+
+            self.save_translation(translation)
+
+
+    def save_translation(self, translation):
+        # Translation models without any fields are also supported.
+        # This is useful for parent objects that have inlines;
+        # the parent object defines how many translations there are.
+        if translation.is_modified or (translation.is_empty and not translation.pk):
+            if not translation.master_id:  # Might not exist during first construction
+                translation.master = self
+            translation.save()
 
 
     def safe_translation_getter(self, field, default=None, any_language=False):
@@ -419,6 +429,10 @@ class TranslatedFieldsModel(models.Model):
     def is_modified(self):
         return self._original_values != self._get_field_values()
 
+    @property
+    def is_empty(self):
+        return len(self.get_translated_fields()) == 0
+
     @property
     def shared_model(self):
         return self.__class__.master.field.rel.to
@@ -461,9 +475,9 @@ class TranslatedFieldsModel(models.Model):
         return [getattr(self, field) for field in self._meta.get_all_field_names()]
 
     @classmethod
-    def get_translated_fields(self):
+    def get_translated_fields(cls):
         # Not using get `get_all_field_names()` because that also invokes a model scan.
-        return [f.name for f, _ in self._meta.get_fields_with_model() if f.name not in ('language_code', 'master', 'id')]
+        return [f.name for f, _ in cls._meta.get_fields_with_model() if f.name not in ('language_code', 'master', 'id')]
 
     @classmethod
     def contribute_translations(cls, shared_model):
